﻿using System.Threading;
using Cysharp.Threading.Tasks;

namespace Magify
{
    public interface IProductObtainer
    {
        public bool ObtainUnderSpin => false;
        public bool CanObtainProduct(ProductDef product);
        public bool NeedPrepareProduct(ProductDef product) => false;
        public UniTask<ProductPrepareResult> PrepareProductAsync(ProductDef product, CancellationToken cancellationToken) => UniTask.FromResult(ProductPrepareResult.Success());
        public UniTask<ProductObtainResult> ObtainProductAsync(ProductDef product, CampaignRequest request);
    }

    public abstract class ProductObtainer<T> : IProductObtainer
        where T : ProductDef
    {
        public virtual bool CanObtainProduct(ProductDef product)
        {
            return product is T;
        }

        public UniTask<ProductObtainResult> ObtainProductAsync(ProductDef product, CampaignRequest request)
        {
            return ObtainProductAsync((T)product, request);
        }

        private protected abstract UniTask<ProductObtainResult> ObtainProductAsync(T product, CampaignRequest request);
    }
}