﻿using System.Collections.Generic;
using System.Linq;
using JetBrains.Annotations;
using Magify.Model;

namespace Magify
{
    internal class AppInstallsTracker : IContextListener
    {
        private readonly AnalyticsTracker _analyticsTracker;
        private readonly GeneralPrefs _generalPrefs;
        private readonly PlatformAPI _platform;

        [CanBeNull]
        private HashSet<string> _trackedAppIds;

        public AppInstallsTracker(AnalyticsTracker analyticsTracker, GeneralPrefs generalPrefs, PlatformAPI platform)
        {
            _analyticsTracker = analyticsTracker;
            _generalPrefs = generalPrefs;
            _platform = platform;
        }

        void IContextListener.UpdateContext(CampaignsContext context, ContextKind kind)
        {
            switch (kind)
            {
                case ContextKind.Default when _trackedAppIds?.Count is 0 or null && context.TrackedAppIds != null:
                case ContextKind.Saved or ContextKind.Downloaded when context.TrackedAppIds != null:
                    _trackedAppIds = context.TrackedAppIds;
                    break;
            }

            if (kind is ContextKind.Downloaded)
            {
                CheckAppInstalls();
            }
        }

        public void CheckAppInstalls()
        {
            if (_trackedAppIds?.Count is 0 or null) return;
            var currentInstalls = _trackedAppIds.Where(appIdentifier => _platform.IsApplicationInstalled(appIdentifier)).ToHashSet();
            if (!currentInstalls.SetEquals(_generalPrefs.TrackedAppInstalls))
            {
                _analyticsTracker.TrackAppInstallEvent(currentInstalls);
                using (_generalPrefs.MultipleChangeScope())
                {
                    _generalPrefs.TrackedAppInstalls.Clear();
                    foreach (var app in currentInstalls)
                    {
                        _generalPrefs.TrackedAppInstalls.Add(app);
                    }
                }
            }
        }
    }
}