﻿using JetBrains.Annotations;
using Newtonsoft.Json;

namespace Magify.Model
{
    internal class PurchaseRecord
    {
#if UNITY_ANDROID
        [JsonProperty("object_id")]
#elif UNITY_IOS
        [JsonProperty("product_identifier")]
#endif
        public string ProductId { get; init; }

        // Android-specific fields
        [JsonProperty("is_subscription")]
        [CanBeNull]
        public bool? IsSubscription { get; init; }

        [JsonProperty("transaction_id")]
        [CanBeNull]
        public string TransactionId { get; init; }

        [JsonProperty("purchase_token")]
        [CanBeNull]
        public string PurchaseToken { get; init; }

        // iOS-specific fields
        [JsonProperty("receipt_data")]
        [CanBeNull]
        public string Receipt { get; init; }

        /// <summary>
        /// Base64 encoded idfv
        /// </summary>
        [JsonProperty("device_identifier")]
        [CanBeNull]
        public string DeviceId { get; init; }

        [JsonProperty("build_id")]
        [CanBeNull]
        public string BuildNumber { get; init; }

        public PurchaseRecord()
        {
        }

        public PurchaseRecord(PurchaseRecord purchaseRecord)
        {
            ProductId = purchaseRecord.ProductId;

            IsSubscription = purchaseRecord.IsSubscription;
            TransactionId = purchaseRecord.TransactionId;
            PurchaseToken = purchaseRecord.PurchaseToken;

            Receipt = purchaseRecord.Receipt;
            DeviceId = purchaseRecord.DeviceId;
            BuildNumber = purchaseRecord.BuildNumber;
        }
    }
}