﻿using System;
using System.Threading.Tasks;
using Cysharp.Threading.Tasks;
using FluentAssertions;
using NUnit.Framework;

namespace Magify.Tests
{
    internal partial class MagifyClientTests
    {
        public class Sync
        {
            [Test]
            [UnitTestTarget(typeof(MagifyClient), nameof(MagifyClient.OnConfigLoaded))]
            // When went to background and return to foreground BEFORE session interval
            [TestCase(0, 2, 0)]
            [TestCase(1, 10, 0)]
            // When went to background and return to foreground AFTER session interval
            [TestCase(2, 1, 1)]
            [TestCase(5, 2, 1)]
            public async Task WhenWentToBackground_AndReturnToForeground_ThenCheckOnConfigLoaded(int delaySeconds, int sessionInterval, int expectedCallbacks)
            {
                // Arrange
                using var client = CreateDefaultAndInit();
                var platform = (PlatformDefault)GetPlatformAPI(client);
                GetSessionCounter(client).Interval.Value = TimeSpan.FromSeconds(sessionInterval);
                var onConfigLoadedCalls = 0;
                client.OnConfigSyncRequested += () => onConfigLoadedCalls++;

                // Act
                platform.GoToBackground();
                if (delaySeconds > 0) await UniTask.WaitForSeconds(delaySeconds);
                platform.ReturnToForeground();

                // Assert
                onConfigLoadedCalls.Should().Be(expectedCallbacks);
            }

            [Test]
            public void WhenLocaleChanged_ThenOnConfigSyncRequestedCalled()
            {
                // Arrange
                using var client = CreateDefaultAndInit();
                var configRequests = 0;
                client.OnConfigSyncRequested += () => configRequests++;

                // Act
                ((PlatformDefault)GetPlatformAPI(client)).TweakUserLocale("rr-RR");

                // Assert
                configRequests.Should().Be(1);
            }
        }
    }
}